import os
import sys
from bson import json_util

current_dir = os.path.dirname(__file__)
parent_dir = os.path.abspath(os.path.join(current_dir, '..'))
sys.path.append(parent_dir)
import json


# get_markers requests all marker data from mongoDB
def get_all_markers(self, cursor):
    try:
        # Fetch all data
        cursor.execute('''
            SELECT m.MeasurementID, m.SensorID, m.TimeMeasured, d.Latitude, d.Longitude,
                   d.IceTop, d.IceBottom, d.CalculatedThickness, d.Accuracy, s.SensorType, s.Active
            FROM Measurement m
            INNER JOIN Sensor s ON m.SensorID = s.SensorID
            INNER JOIN Data d ON m.MeasurementID = d.MeasurementID
        ''')

        rows = cursor.fetchall()

        measurement_data = {}

        for row in rows:
            measurement_id = row[0]
            data_object = {
                'Latitude': row[3],
                'Longitude': row[4],
                'IceTop': row[5],
                'IceBottom': row[6],
                'CalculatedThickness': row[7],
                'Accuracy': row[8]
            }

            # Append data with measurement ID x to measurement x object
            if measurement_id in measurement_data:
                measurement_data[measurement_id]['Data'].append(data_object)
            else:
                # Create a new measurement entry if measurement ID doesn't exist already
                measurement_data[measurement_id] = {
                    'MeasurementID': measurement_id,
                    'TimeMeasured': row[1],
                    'Sensor': {
                        'SensorID': row[2],
                        'SensorType': row[9],
                        'Active': bool(row[10])
                    },
                    'Data': [data_object]
                }

        # Convert dictionary values to list of measurements
        data = list(measurement_data.values())

        if len(rows) == 0 or len(data) == 0: # Return 500 and empty list if no data is found
            print(f"An error occurred while querying the database")
            resp_code = 500
            marker_json = '[]'
        else:
            resp_code = 200
            # Convert list of dictionaries to JSON
            marker_json = json.dumps(data, indent=4)

    except Exception as e:
        print(f"An error occurred while querying the database: {e}")
        resp_code = 500
        marker_json = '[]'

    # Set headers
    self.send_response(resp_code)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    # Write marker_data to the response object
    self.wfile.write(marker_json.encode('utf-8'))
